;	>>> Beam Racer by JAC! <<<
;
;	Created 2011-04-01/2011-04-23 for Revision 2011 in Saarbrcken.
;	V1.1 with faster text scrolling and support to RESET button.
;	V1.2 with correct VSYNC (312/262 scanlines) and VBLANK at all times.
;	V1.3 with correct silence (not 15 kHz bug via AUDCx, AUDVx is 0 now).
;
; Type         NTSC  PAL/SECAM
;  V-Sync      3     3    scanlines
;  V-Blank     37    45   scanlines (upper border)
;  Picture     192   228  scanlines
;  Overscan    30    36   scanlines (lower border)
;  Frame Rate  60    50   Hz
;  Frame Time  262   312  scanlines

;	Set to 0 for NTSC version and to 1 for PAL vesion
pal	= 0;

;	Set to 0 for fast animations
text_delay	= 0;
curtain_delay	= 1;
bar_delay	= 1;

	icl "../VCS.asm"


; Constant declarations

	
	.if pal = 0
	.print "Using NTSC colors"
	.enum color
	yellow = $10
	red = $40
	violet = $50
	blue_violet = $60
	blue_seven = $70
	blue = $80
	blue_green = $c0
	green = $d0
	olive = $e0
	gold = $f0
	.ende
	.else
	.print "Using PAL colors"
	.enum color
	yellow = $20
	red = $60
	violet = $a0
	blue_violet = $c0
	blue_seven = $b0
	blue = $d0
	blue_green = $90
	green = $50
	olive = $40
	gold = $20
	.ende
	.endif

; Zeropage variable declaration

		org $80	;Zeropage variables
cnt		.ds 1
sound_zp	.ds 3	;3 bytes

plot_cnt_hi	.ds 1	;Plot counter

text_cnt_lo	.ds 2	;Up and down scroll counter
text_cnt_hi	.ds 2	;Text phase counter

curtain_cnt_lo	.ds 1	;Up and down scroll counter
curtain_cnt_hi	.ds 1	;Phase counter: bars up, players up, bars players down, bars down

bar_cnt_sin1	.ds 1
bar_cnt_sin2	.ds 1
bar_cnt_lo	.ds 1	;$00..$ef (all bars off to 15 bar on)

volatile_zp	.ds 6

effect_lines	= $68
effect_tab	.ds effect_lines

zp_end
	.if zp_end >$fc
		.error "zp_end=",zp_end
	.endif

;===============================================================

	opt h-f+

	org $f000		;Main part

	icl "BeamRacer-Sound.asm"

;===============================================================

	.proc cart
start	cld
	ldx #0
	txa
;	tay
init
	dex
	txs
	pha
	bne init		;SP=$FF, X = A = 0

	mva #text_bottom-text_top text_cnt_hi+1	;Start of 2nd line text

;===============================================================

	.proc main
frame_loop
	lsr swchb		;Reset pressed?
	bcc start		;Yes, reset

	ldx #$02
	sta wsync
	stx vsync
	sta wsync
	ldx #1			;Set horizontal player position
	jsr delay
	sta resp0
	ldx #5
	jsr delay
	bit $00
	sta resp1
	sta wsync
	sta wsync
	sta vsync

	.proc vblank
	.if pal = 0 
	lda #45
	.else
	lda #74
	.endif
	sta tim64t

	lda plot_cnt_hi
	cmp #animate.plot.bar_start
	bcc no_bar_move
	cmp #animate.plot.bar_end+1
	bcs no_bar_move
	jsr bar.move
no_bar_move

loop	lda intim
	bne loop
	sta wsync
	sta :vblank		;Deactivate VBLANK bit 1
	.endp

;===============================================================
	ldx #0			;Begin of playfield
	jsr text_box.kernel

	sty pf0
	sty pf1
	sty pf2

	sta wsync		;Top splitt sync 1
	sty colubk		;Top splitter black

	ldx cnt			;Preset bar animation variables
	lda sinus,x
	sta bar.kernel.bak_sin	;Background color y sinus value
	lda cnt
	sta bar.kernel.plr_sin	;Player graphics y sinus value

	ldx plot_cnt_hi
	lda bar_table-animate.plot.bar_start,x
	tax
	lda bar.bar_player_add,x;Player graphics y sinus add value
	sta bar.kernel.plr_sin_add

	sta wsync		;Top splitt sync 2
	lda #14			;Top splitter white
	sta colubk

	jsr animate.playfield

	sta wsync		;Bottom splitt sync 1
	lda #14			;Bottom splitter white 
	sta colubk
	lda #0
	sta pf0
	sta pf1
	sta pf2
	sta grp0
	sta grp1

	lda #4			;Set flashing bottom text color
	jsr flash
	sta colupf

	sta wsync		;Bottom splitt sync 2
	lda #0			;Bottom splitter black
	sta colubk
	sta ctrlpf		;No reflect

	ldx #1
	jsr text_box.kernel
	sty colubk		;Overscan black
	sty pf0
	sty pf1
	sty pf2

;===============================================================
	.proc overscan
	lda #2
	sta :vblank		;Activate VBLANK bit 1
	.if pal = 0
	lda #34
	.else
	lda #65
	.endif
	sta tim64t		;Overscan time counter
	
	lda #0			;Set flashing player 0 and 1 colors
	jsr flash
	sta colup0
	lda #8
	jsr flash
	sta colup1
	
	lda #12			;Set flashing top text color
	jsr flash
	sta colupf

	jsr sound.player	;Play sound and returns C=1 if main pattern has changed
	jsr animate.overscan	;Animate all effects

	inc cnt
loop	lda intim
;stop	bmi stop		;Time overrun
	bne loop
	sta wsync
	jmp frame_loop
	
	.endp			;End of overscan
	.endp			;End of main

;===============================================================
	.proc delay
	dex
	bne delay
	.endp
	nop
nothing	rts
	
;===============================================================
	.proc flash
	asl
	adc cnt
	lsr
	and #31
	cmp #16
	bcc *+4
	eor #31
	rts
	.endp

;===============================================================
;	Display text box
;
;	IN : <X> will hold the index of letter pixel we're on couting in lines
;	OUT: <Y> = 0

	.proc text_box

;	Local equates
	text_lines 		= 16
	text_line_height 	= 5
	text_pixel_per_line	= 8
	text_pixel_height	= text_line_height*text_pixel_per_line;
	text_column_offset	= text_lines*text_line_height;

;	Local zero page
	text_temp		= volatile_zp
	text_pixel_counter	= volatile_zp+1
	text_color		= volatile_zp+2

;	IN: <X> = 0/1
	.proc up
	lda text_cnt_lo,x
	cmp #text_pixel_height
	beq no_inc
	.if text_delay <> 0
	lda cnt
	and #text_delay
	bne no_inc
	.endif
	inc text_cnt_lo,x
no_inc	rts
	.endp

;	IN: <X> = 0/1
	.proc down
	lda text_cnt_lo,x
	beq no_dec
	.if text_delay <> 0
	lda cnt
	and #text_delay
	bne no_dec
	.endif
	dec text_cnt_lo,x
	bne no_next
	inc text_cnt_hi,x
no_next
no_dec
	rts
	.endp

	.proc kernel
	sec
	lda #text_pixel_height
	sbc text_cnt_lo,x		;<X>=0/1
	beq no_skip
	tay
skip_loop
	sta wsync
	dey
	bne skip_loop

no_skip	cmp #text_pixel_height
	bcs skip_all			;Full invisible
	adc #<text_background		;C=0
	tay
	mva #text_pixel_per_line text_pixel_counter

	lda text_cnt_hi,x		;<X>=0/1
	tax	
	lda text_colors,x
	sta text_color
	lda text_top,x
	tax

loop	lda text_background_page,y	;Y is index in page
	eor text_color
	sta wsync
	sta colubk	;3 cylces
loop_with_new_line
	lda texts1+text_column_offset*0,x
	sta pf0
	lda texts1+text_column_offset*1,x
	sta pf1
	lda texts1+text_column_offset*2,x
	sta pf2
	bit $00		;3 cycles
	nop
	lda texts2+text_column_offset*0,x
	sta pf0
	lda texts2+text_column_offset*1,x
	sta pf1
	lda texts2+text_column_offset*2,x
	sta pf2
	dec text_pixel_counter
	bne same_line
	mva #text_pixel_per_line text_pixel_counter
	inx
	sta wsync
	iny
	bne loop_with_new_line
	rts

same_line
	iny
	bne loop
skip_all
	sta wsync
	rts
	.endp			;End of kernel
	
	.print "PROC text_box.kernel:", text_box, " - ", text_box+.len text_box-1, " (", .len text_box, " bytes)"
	
	.endp			;End of text_box

;===============================================================
	.proc intro

	.proc kernel

;	Local zero page
	pattern = volatile_zp
	line_cnt = volatile_zp+1
	
	.proc none
	ldx #effect_lines+1
	jmp top
	.endp

	.proc rom
	lda text_cnt_lo+1	;Position of lower text
	asl			;C=0
	adc text_cnt_lo+1	;<A>=position*3
	cmp #effect_lines+1
	bcc ok
	lda #effect_lines+1
ok	sta line_cnt
	tax
	beq skip_all

	lda #color.green+$0a
	sta colupf

	ldy #0
	lda #$55
loop	sta wsync
	sty colubk
	sta pf0
	sta pf2
	eor #$ff
	sta pf1
	dex
	bne loop
skip_all
	ldx line_cnt
	jmp bottom
	.endp

	.proc ram
	ldx #20
	jsr top
	lda text_cnt_lo+1	;Position of lower text
	asl			;<A>=position*2
	cmp #64
	bcc ok
	lda #64
ok	sta line_cnt
	tax
	bne do_it
	sta wsync
	beq skip_all
	
do_it	lda #color.yellow+$0a
	sta colupf
	mva #1 ctrlpf		;Reflect playfield

	ldy #7
	lda #$55
	sta pattern
loop	sta wsync
	lda #0
	sta colubk
	sta pf0

	lda pattern
	sta pf1
	eor #$ff
	sta pf2
;	The toggle sequence must have 24 cycles
	dey		;2 cycles
	bpl no_toggle	;2/3 cycles
	ldy #7		;2 cycles
	sta pattern	;3 cycles
	bne toggle	;3 cycles
no_toggle
	pha		;3 cycles
	pla		;16 cycles
toggle	pha		;12 cycles	
	pla
	sta cxclr
	nop
	
	eor #$ff
	sta pf2
	eor #$ff
	sta pf1

	dex
	bne loop
	stx colupf

skip_all
	lda line_cnt
	clc
	adc #21
	tax
	jmp bottom
	.endp
	
	.proc screen
	ldx #45
	jsr top
	lda text_cnt_lo+1	;Position of lower text
	cmp #16
	bcc ok
	lda #16
ok	sta line_cnt
	tax
	beq skip_all

	lda #color.red+$0a
	sta colupf
	mva #1 ctrlpf		;Reflect playfield

	ldy #15
	lda cnt
	and #8
	beq flash
	lda #$b0
flash	sta pattern

loop	sta wsync
	lda #0
	sta colubk
	sta pf0
	sta pf1
	lda pattern
	sta pf2
:12	nop
	and #$e0
	sta pf2
	dex
	bne loop
skip_all
	lda line_cnt
	clc
	adc #45
	tax
	jmp bottom
	.endp

	.proc top
	lda #0
	cpx #0			;No lines to display
	beq exit
loop	sta wsync
	sta colubk
	sta colupf
	dex
	bne loop
exit	rts
	.endp			;End of top


	.proc bottom
	lda #0
loop	cpx #effect_lines+1	;No more lines to display
	bcs exit 
	sta wsync
	sta colubk
	sta colupf
	inx
	bne loop
exit	rts
	.endp			;End of bottom

	.endp			;End of kernel

	.endp			;End of intro

;===============================================================

	.proc curtain

;	IN: <A> = $00,$80,$C0
	.proc init
;	Clear complete effect table
	ldx #(effect_lines/2)-1
clear_tab
	sta effect_tab,x
	sta effect_tab+effect_lines/2,x
	dex
	bpl clear_tab
	rts
	.endp

	.proc move
	lda plot_cnt_hi
	ldy curtain_cnt_hi
	cmp #animate.plot.curtain_up
	beq phase_up
	cmp #animate.plot.curtain_down
	beq phase_down

phase_up
	cpy #2
	beq no_hi
	bne phase_ok

phase_down
	cpy #4
	beq no_hi
	bne phase_ok

phase_ok
	lda curtain_cnt_lo
	cmp #effect_lines
	beq no_lo

	cpy #1
	beq no_invert
	cpy #2
	beq no_invert
	lda #effect_lines-1
	sec
	sbc curtain_cnt_lo
no_invert
	tax
	lda bitmask,y
	sta effect_tab,x
	lda cnt
	and #curtain_delay
	bne no_up
	inc curtain_cnt_lo
no_up	rts

no_lo	inc curtain_cnt_hi
	lda #0
	sta curtain_cnt_lo
no_hi	rts

bitmask	.byte $80,$c0,$80,$00

	.endp			;End of up

	.proc kernel
;	Local variables
	bak_sin = volatile_zp+2
	plr_sin = volatile_zp+3

	ldx #effect_lines-1
loop	ldy bak_sin
	lda pattern_background-1,y
	sta wsync
	ldy effect_tab,x	;Bit 7 indicates that backgound shall be animated
	bmi with_bak
	lda #0
with_bak
	sta colubk
	inc bak_sin

	ldy plr_sin
	txa
	clc
	adc sinus,y
	tay
	lda effect_tab,x	;Bit 6 indicates that backgound players be animated
	and #$40
	beq without_player
	lda players,y
without_player
	sta grp0
	sta grp1

	dex
	bpl loop
	sta wsync
	rts
	.endp			;End of kernel

	.print "PROC curtain.kernel:", kernel, " - ", kernel+.len kernel-1, " (", .len kernel, " bytes)"
	assert_same_page kernel
	.endp			;End of player

;===============================================================

	.proc bar

bar_speed1	.byte $01,$01,$02,$01
bar_add1	.byte $10,$18,$01,$0f
bar_speed2	.byte $01,$02,$03,$01
bar_add2	.byte $10,$09,$1d,$21

bar_player_add	.byte $00,$01,$02,$00

;===============================================================
	.proc up		;Fade bars up
	lda bar_cnt_lo
	cmp #$ef
	beq no_bar
	cmp #16
	bne no_first
no_first
	and #$0f
	cmp #$0f
	bne do_bar
	lda pattern_lo
	bne no_bar
	lda cnt
	and #bar_delay
	bne no_bar
do_bar	inc bar_cnt_lo
no_bar	rts
	.endp			;End of up

;===============================================================
	.proc down		;Fade bars down
	lda bar_cnt_lo
	beq no_bar
	lda cnt
	and #bar_delay
	bne no_bar
do_bar	dec bar_cnt_lo
no_bar	rts
	.endp			;End of down

;===============================================================
	.proc move		;Animate bars

;	Local zero page
cur_bar		= volatile_zp	;$00..$ef
max_bar		= volatile_zp+1	;$00/$10/...$e0
cur_sin1	= volatile_zp+2
cur_add1	= volatile_zp+3
cur_sin2	= volatile_zp+4
cur_add2	= volatile_zp+5

;	Clear complete effect table
	ldx #(effect_lines/2)-1
	lda #0
clear_tab
	sta effect_tab,x
	sta effect_tab+effect_lines/2,x
	dex
	bpl clear_tab

;	Compute the sinus counter
	ldx plot_cnt_hi
	lda bar_table-animate.plot.bar_start,x
	tax
	lda bar_cnt_sin1
	clc
	adc bar_speed1,x
	sta bar_cnt_sin1
	sta cur_sin1
	lda bar_add1,x
	sta cur_add1

	lda bar_cnt_sin2
	clc
	adc bar_speed2,x
	sta bar_cnt_sin2
	sta cur_sin2
	lda bar_add2,x
	sta cur_add2

	lda bar_cnt_lo
	and #$f0
	sta max_bar
	cmp #$f0
	beq no_bar

	mva #$f0 cur_bar
fill_loop
	clc
	lda cur_sin1
	adc cur_add1
	sta cur_sin1
	tax
	lda cur_sin2
	adc cur_add2
	sta cur_sin2
	tay
	clc
	lda sinus,x
	adc sinus,y
	lsr
	tax

	lda cur_bar
	clc
	adc #$10
	sta cur_bar
	cmp max_bar
	beq fade
	bcs fill_exit
	lda cur_bar		;Less, so bar is full visible
	ora #2
	bne line5

fade	lda bar_cnt_lo		;High nibble is equal, so fade in
	and #15
	tay
	lda cur_bar		;High nibble is color
	ora #2
	cpy #0
	beq line0
	cpy #3
	bcc line1
	cpy #5
	bcc line2
	cpy #7
	bcc line3
	cpy #9
	bcc line4

line5	sta effect_tab,x
	sta effect_tab+9,x
	adc #2
line4	sta effect_tab+1,x
	sta effect_tab+8,x
	adc #2
line3	sta effect_tab+2,x
	sta effect_tab+7,x
	adc #2
line2	sta effect_tab+3,x
	sta effect_tab+6,x
	adc #2
line1	sta effect_tab+4,x
	sta effect_tab+5,x
line0	jmp fill_loop

fill_exit
no_bar
	rts
	.endp			;End of move

before_align
;	.align 256
after_align
	.print "PROC bar.align:", after_align-before_align, " bytes)"
;===============================================================

	.proc kernel
;	Local zero page
bak_sin = volatile_zp+2
plr_sin = volatile_zp+3
plr_sin_add = volatile_zp+5

	mva #1 ctrlpf		;Reflect playfield
	ldx #effect_lines-1
	ldy bak_sin
	lda pattern_background-1,y
	sta wsync

loop	sta colubk
	ldy effect_tab,x
	sty colupf
	lda pattern_idx,y
	tay
	lda pattern_pf1,y
	sta pf1
	lda pattern_pf2,y
	sta pf2
	inc bak_sin
	lda plr_sin
;	clc			;Ignore carry errors
	adc plr_sin_add
	sta plr_sin
	tay
	txa
;	clc			;Ignore carry errors
	adc sinus,y
	tay
	lda players,y
	sta grp0
	sta grp1

	ldy bak_sin
	lda pattern_background-1,y;Make sure pattern_background-1 does not cross a page boundary
	dex
	bpl loop
	rts
	.endp			;End of kernel

	.print "PROC bar.kernel:", kernel, " - ", kernel+.len kernel-1, " (", .len kernel, " bytes)"
	assert_same_page kernel kernel
	.endp			;End of bar

;===============================================================

	.proc animate

;===============================================================

	.proc playfield		;Animation code to be execute for the  playfield
	jump_ptr = volatile_zp	;Must not be overwritte here

	lda plot_cnt_hi
	asl
	tax
	lda playfield_table,x
	sta jump_ptr
	lda playfield_table+1,x
	sta jump_ptr+1
	jmp (jump_ptr)
	.endp			

;===============================================================

	.proc overscan		;Animation code to be executed during VBLANK
;	Local zero page
	init_flag = volatile_zp
	jump_ptr = volatile_zp+1

	lda #0
	sta init_flag
	bcc same_pattern	;Same pattern

	ldx #.len pattern_table-1
	lda pattern_hi
pattern_loop
	cmp pattern_table,x	;Pattern which increases plot?
	beq next_plot		;Yes
	dex
	bpl pattern_loop
	bmi same_pattern
next_plot
	inc plot_cnt_hi
	inc init_flag
same_pattern
	ldx plot_cnt_hi
	txa
	asl
	tax
	lda overscan_table,x
	sta jump_ptr
	lda overscan_table+1,x
	sta jump_ptr+1
	lda init_flag
	jmp (jump_ptr)		;Start with Z=0 in case of INIT
	.endp			;End of overscan


text_box_up_1
	ldx #0
	.byte $2c		;Skip next 2 bytes
text_box_up_2
	ldx #1
	jmp text_box.up

text_box_down_12
	ldx #0
	jsr text_box.down
text_box_down_2
	ldx #1
	jmp text_box.down

curtain_down
	beq curtain_down_no_init
	lda #$c0
	jsr curtain.init
curtain_down_no_init
	jmp curtain.move

;===============================================================

	.local pattern_table

	.byte $08,$0a,$0c	;4K ROM/OK
	.byte $0e,$10,$12	;RAM/OOPS..
	.byte $14,$16,$18	;SCREEN/OH SHIT!
	.byte $1c,$1e		;Code/JAC
	.byte $20		;Curtain up
	.byte $28		;Bars up
	.byte $38,$3a,$3c	;Sound/Qotile
	.byte $40		;Bars down
	.byte $48		;Bars up
	.byte $58		;Text down
	.byte $5a,$5c		;Hello/Noice
	.byte $5e,$60		;Hello/Trilobit
	.byte $62,$64		;Hello/Wamma
	.byte $66		;Hello down
	.byte $68		;Bars down
	.byte $70,$72		;Atari 1977/Revision 2011
	.byte $74		;Curtain down
	.byte $79		;Stop
	.endl

;===============================================================

	.enum plot		;Count value of plot_cnt_hi
	curtain_up = 12
	bar_start = 13
	bar_end = 29
	curtain_down = 30
	.ende

;===============================================================

	.local playfield_table
	.word intro.kernel.none
	.word intro.kernel.none,intro.kernel.rom,intro.kernel.rom	;ROM - $08,$0a,$0c
	.word intro.kernel.none,intro.kernel.ram,intro.kernel.ram	;RAM - $0e,$10,$12
	.word intro.kernel.none,intro.kernel.screen,intro.kernel.screen	;SCREEN - $14,$16,$18
	.word intro.kernel.none,intro.kernel.none			;Code/JAC - $1c,$1c
	.word curtain.kernel						;Curtain up - $20
	.word bar.kernel						;Bars up - $28
	.word bar.kernel,bar.kernel,bar.kernel				;Sound/Qotile - $38
	.word bar.kernel						;Bars down - $40
	.word bar.kernel						;Bars up - $48
	.word bar.kernel						;Text down - $58
	.word bar.kernel,bar.kernel					;Hello/Noice - $5a,$5c
	.word bar.kernel,bar.kernel					;Hello/Trilobit - $5e,$60
	.word bar.kernel,bar.kernel					;Hello/Wamma - $62,$64
	.word bar.kernel						;Text down - $66
	.word bar.kernel						;Bars down - $68
	.word bar.kernel,bar.kernel					;Atari 1977/Revision 2011 - $70,$72
	.word curtain.kernel						;Curtain down - $74
	.word intro.kernel.none
	.endl
	
;===============================================================

	.local overscan_table
	.word nothing
	.word text_box_up_1,text_box_up_2,text_box_down_12	;ROM
	.word text_box_up_1,text_box_up_2,text_box_down_12	;RAM
	.word text_box_up_1,text_box_up_2,text_box_down_12	;SCREEN
	.word text_box_up_1,text_box_up_2			;Code/JAC
	.word curtain.move					;Curtain up
	.word bar.up						;Bars up
	.word text_box_down_12,text_box_up_1,text_box_up_2	;Sound/Qotile
	.word bar.down						;Bars down
	.word bar.up						;Bars up
	.word text_box_down_12					;Text down
	.word text_box_up_1,text_box_up_2			;Hello/Noice
	.word text_box_down_2,text_box_up_2			;Hello/Trilobit
	.word text_box_down_2,text_box_up_2			;Hello/Wamma
	.word text_box_down_12					;Text down
	.word bar.down						;Bars down
	.word text_box_up_1, text_box_up_2			;Atari 1977/Revision 2011 
	.word curtain_down					;Curtain down
	.word stop						;Stop
	.endl

;===============================================================

	.proc stop
	lda #0
	sta audv0
	sta audv1
wait	lda #2
	sta vblank
	lsr swchb 		;Reset pressed?
	bcs wait		;No, wait
	jmp ($fffc)		;Yes, Restart
	.endp			;End of stop

;===============================================================

	.print "LOCAL pattern_table:	", pattern_table, " - ", pattern_table+.len pattern_table-1, " (", .len pattern_table, " bytes)"
	.print "LOCAL overscan_table:	", overscan_table, " - ", overscan_table+.len overscan_table-1, " (", .len overscan_table, " bytes)"
	.print "LOCAL playfield_table:	", playfield_table, " - ", playfield_table+.len playfield_table-1, " (", .len playfield_table, " bytes)"
	
	.endp			;End of animate
	
;===============================================================

	.endp			;End of cart

	.print "PROC cart:	", cart, " - ", cart+.len cart-1, " (", .len cart, " bytes)"
	.print "Remaining:	", $fa00-cart-.len cart , " bytes"


;===============================================================
	org $fa00		;Two pages with 3 text columns each
texts1	ins "BeamRacer-Text.bin",+$00,$f0

text_top	.byte 0,10,20,30,40,50,70
text_bottom	.byte 5,15,25,35,45,55,60,65,75

texts2	ins "BeamRacer-Text.bin",+$f0,$f0

text_colors	.byte color.blue,color.blue,color.blue,color.red,color.violet,color.gold,color.blue_seven
		.byte color.green,color.yellow,color.red,color.blue,color.blue_violet,color.olive,color.green,color.blue_green,color.blue

;===============================================================
	org $fc00		;One page player graphics including padding
text_background_page

players
:21	.byte $00			;Zero padding
	ins "BeamRacer-Players.bin"	;$9b bytes
:21	.byte $00			;Zero padding

bar_table
	.byte 0,0,0,0,0,1,1,1,1,1,1,1,1,1,1	;here are some free bytes

	org $fd00-40		;40 bytes of text background up to end of page
text_background			;Background colors
	.byte $0f,$0f,$0f,$0f,$0f,$0f,$0f,$0f
	.byte $0d,$0f,$0d,$0d,$0b,$0d,$0b,$0b
	.byte $09,$0b,$09,$09,$07,$09,$07,$07
	.byte $05,$07,$05,$05,$03,$05,$03,$03
	.byte $00,$00,$00,$00,$00,$00,$00,$00

;===============================================================
	org $fd00		;One page sinus table
sinus	dta b(sin($2f,$2e,256));Sinus table

;===============================================================
	org $fe00		;One page of width to pattern translation table
pattern_idx			;
	.rept 256
	.byte #/16
	.endr

;===============================================================
	org $ff00		;One page of pattern data

	.local pattern_pf1			;Playfield 1 pattern
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$03,$07,$0f,$1f,$3f,$7f,$ff
	.endl

	.local pattern_pf2			;Playfield 2 pattern
	.byte $00,$80,$c0,$e0,$f0,$f8,$fc,$fe,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff
	.endl

	.local pattern_background		;Background colors
	color1 = color.blue;
	color2 = color.red;

	.byte $0f,$0f,color1+$0f,$0f,color1+$0f,color1+$0f,color1+$0f,color1+$0f
	.byte color1+$0d,color1+$0f,color1+$0d,color1+$0d,color1+$0b,color1+$0d,color1+$0b,color1+$0b
	.byte color1+$09,color1+$0b,color1+$09,color1+$09,color1+$07,color1+$09,color1+$07,color1+$07
	.byte color1+$05,color1+$07,color1+$05,color1+$05,color1+$03,color1+$05,color1+$03,color1+$03
	.byte color2+$03,color2+$05,color2+$03,color2+$03,color2+$05,color2+$07,color2+$05,color2+$05
	.byte color2+$07,color2+$09,color2+$07,color2+$07,color2+$09,color2+$0b,color2+$09,color2+$09
	.byte color2+$0b,color2+$0d,color2+$0b,color2+$0b,color2+$0d,color2+$0f,color2+$0d,color2+$0d
	.byte color2+$0f,$0f,color2+$0f,color2+$0f,$0f,$0f,$0f,$0f

	.byte $0f,$0f,color1+$0f,$0f,color1+$0f,color1+$0f,color1+$0f,color1+$0f
	.byte color1+$0d,color1+$0f,color1+$0d,color1+$0d,color1+$0b,color1+$0d,color1+$0b,color1+$0b
	.byte color1+$09,color1+$0b,color1+$09,color1+$09,color1+$07,color1+$09,color1+$07,color1+$07
	.byte color1+$05,color1+$07,color1+$05,color1+$05,color1+$03,color1+$05,color1+$03,color1+$03
	.byte color2+$03,color2+$05,color2+$03,color2+$03,color2+$05,color2+$07,color2+$05,color2+$05
	.byte color2+$07,color2+$09,color2+$07,color2+$07,color2+$09,color2+$0b,color2+$09,color2+$09
	.byte color2+$0b,color2+$0d,color2+$0b,color2+$0b,color2+$0d,color2+$0f,color2+$0d,color2+$0d
	.byte color2+$0f,$0f,color2+$0f,color2+$0f,$0f,$0f,$0f,$0f

	.byte $0f,$0f,color1+$0f,$0f,color1+$0f,color1+$0f,color1+$0f,color1+$0f
	.byte color1+$0d,color1+$0f,color1+$0d,color1+$0d,color1+$0b,color1+$0d,color1+$0b,color1+$0b
	.byte color1+$09,color1+$0b,color1+$09,color1+$09,color1+$07,color1+$09,color1+$07,color1+$07
	.byte color1+$05,color1+$07,color1+$05,color1+$05,color1+$03,color1+$05,color1+$03,color1+$03
	.byte color2+$03,color2+$05,color2+$03,color2+$03,color2+$05,color2+$07,color2+$05,color2+$05
	.byte color2+$07,color2+$09,color2+$07,color2+$07,color2+$09,color2+$0b,color2+$09,color2+$09
	.byte color2+$0b,color2+$0d,color2+$0b,color2+$0b,color2+$0d,color2+$0f,color2+$0d,color2+$0d
	.byte color2+$0f,$0f,color2+$0f,color2+$0f,$0f,$0f,$0f,$0f

	.byte $0f,$0f,color1+$0f,$0f,color1+$0f,color1+$0f,color1+$0f,color1+$0f
	.endl

;===============================================================
	.if pal = 0
	.byte 'BeamRacer 1.3 NTSC'
	.else
	.byte 'BeamRacer 1.3 PAL '
	.endif

	org $fffc		;Cartridge vectors
	.word cart.start	;Reset vector
	.word cart.start	;IRQ vector, not really required
